' Functional Dependency Application
' Written by David J. Stein, Esq., Kevin Hardy, and Craig White. See "License.html" for terms and conditions of use.
'
' CheckBCNFStatus - written by David J. Stein, Esq.
' This module checks a functional dependency set for Boyce-Codd normal form.

Public Class CheckBCNFStatus
 Inherits System.Windows.Forms.Form

#Region "Variables and initialization code"

 Dim Attributes As ArrayList
 Dim FunctionalDependencies As ArrayList
 Private x As Integer
 Private y As Integer

#Region " Windows Form Designer generated code "

 Public Sub New(ByRef Attributes As ArrayList, ByRef FunctionalDependencies As ArrayList, ByVal x As Integer, ByVal y As Integer)
  MyBase.New()
  InitializeComponent()
  Me.x = x
  Me.y = y
  Me.Attributes = Attributes
  Me.FunctionalDependencies = FunctionalDependencies
 End Sub

 'Form overrides dispose to clean up the component list.
 Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
  If disposing Then
   If Not (components Is Nothing) Then
    components.Dispose()
   End If
  End If
  MyBase.Dispose(disposing)
 End Sub

 'Required by the Windows Form Designer
 Private components As System.ComponentModel.IContainer

 'NOTE: The following procedure is required by the Windows Form Designer
 'It can be modified using the Windows Form Designer.  
 'Do not modify it using the code editor.
 Friend WithEvents Label1 As System.Windows.Forms.Label
 Friend WithEvents btnOK As System.Windows.Forms.Button
 Friend WithEvents lstResults As System.Windows.Forms.ListBox
 <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
  Dim resources As System.Resources.ResourceManager = New System.Resources.ResourceManager(GetType(CheckBCNFStatus))
  Me.Label1 = New System.Windows.Forms.Label
  Me.btnOK = New System.Windows.Forms.Button
  Me.lstResults = New System.Windows.Forms.ListBox
  Me.SuspendLayout()
  '
  'Label1
  '
  Me.Label1.Location = New System.Drawing.Point(49, 8)
  Me.Label1.Name = "Label1"
  Me.Label1.Size = New System.Drawing.Size(232, 23)
  Me.Label1.TabIndex = 0
  Me.Label1.Text = "BCNF Status:"
  Me.Label1.TextAlign = System.Drawing.ContentAlignment.MiddleCenter
  '
  'btnOK
  '
  Me.btnOK.Location = New System.Drawing.Point(128, 416)
  Me.btnOK.Name = "btnOK"
  Me.btnOK.TabIndex = 2
  Me.btnOK.Text = "OK"
  '
  'lstResults
  '
  Me.lstResults.Location = New System.Drawing.Point(8, 40)
  Me.lstResults.Name = "lstResults"
  Me.lstResults.SelectionMode = System.Windows.Forms.SelectionMode.None
  Me.lstResults.Size = New System.Drawing.Size(312, 368)
  Me.lstResults.TabIndex = 3
  '
  'CheckBCNFStatus
  '
  Me.AutoScaleBaseSize = New System.Drawing.Size(5, 13)
  Me.ClientSize = New System.Drawing.Size(330, 448)
  Me.Controls.Add(Me.lstResults)
  Me.Controls.Add(Me.btnOK)
  Me.Controls.Add(Me.Label1)
  Me.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle
  Me.Icon = CType(resources.GetObject("$this.Icon"), System.Drawing.Icon)
  Me.MaximizeBox = False
  Me.MinimizeBox = False
  Me.Name = "CheckBCNFStatus"
  Me.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent
  Me.Text = "Check BCNF Status"
  Me.ResumeLayout(False)

 End Sub

#End Region

#End Region

#Region "Button-click event handlers"

 Private Sub btnOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnOK.Click
  Close()
 End Sub
 Private Sub CheckBCNFStatus_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
  Dim ImproperFDs As ArrayList = Check(Attributes, FunctionalDependencies)
  Me.Location = New Point(x, y)
  lstResults.BeginUpdate()
  If ImproperFDs.Count = 0 Then
   lstResults.Items.Add("This relation is BCNF.")
  Else
   lstResults.Items.Add("This relation is not BCNF due to the following non-trivial FDs:")
   For Each FD As FunctionalDependency In ImproperFDs
    Dim fdString As String = ""
    For Each LHSItem As Integer In FD.LeftHandSide
     fdString += Attributes(LHSItem) + " "
    Next
    fdString += "-> "
    For Each RHSItem As Integer In FD.RightHandSide
     fdString += Attributes(RHSItem) + " "
    Next
    lstResults.Items.Add(fdString)
   Next
  End If
  lstResults.EndUpdate()
 End Sub

#End Region

#Region "Core functions"

 Public Function Check(ByRef Attributes As ArrayList, ByRef FunctionalDependencies As ArrayList) As ArrayList
  Dim ImproperFDs As New ArrayList
  Dim Lin As New LinClosure
  For Each FD As FunctionalDependency In FunctionalDependencies
   ' Check to see if it's trivial
   Dim Trivial As Boolean = True
   For Each RHS As Integer In FD.RightHandSide
    Dim Found As Boolean = False
    For Each LHS As Integer In FD.LeftHandSide
     If LHS = RHS Then
      Found = True
     End If
    Next
    If Found = False Then
     Trivial = False
    End If
   Next
   If Trivial = False Then
    Dim DeterminedAttributes As ArrayList = Lin.ComputeLinClosure(Attributes, FD.LeftHandSide, FunctionalDependencies)
    If DeterminedAttributes.Count < Attributes.Count Then
     ImproperFDs.Add(FD)
    End If
   End If
  Next
  Return ImproperFDs
 End Function

#End Region

End Class
