' Functional Dependency Application
' Written by David J. Stein, Esq., Kevin Hardy, and Craig White. See "License.html" for terms and conditions of use.
'
' CheckRedundancy - written by Kevin Hardy
' This module checks a functional dependency set for (non)redundancy.

Public Class CheckRedundancy
 Inherits System.Windows.Forms.Form

#Region "Variables and initialization code"

 Private Attributes As ArrayList
 Private FunctionalDependencies As ArrayList
 Private ReducedFunctionalDependencies As ArrayList
 Public ReturnedFunctionalDependencies As ArrayList
 Private x As Integer
 Private y As Integer

 Private Sub CheckRedundancy_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
  Location = New Point(x, y)
 End Sub

#Region " Windows Form Designer generated code "

 Public Sub New(ByVal Attributes As ArrayList, ByVal FunctionalDependencies As ArrayList, ByVal x As Integer, ByVal y As Integer)
  MyBase.New()
  InitializeComponent()
  Me.x = x
  Me.y = y
  Me.Attributes = Attributes
  Me.FunctionalDependencies = FunctionalDependencies
  ReturnedFunctionalDependencies = FunctionalDependencies
  Dim c As New CheckRedundancyObject
  ReducedFunctionalDependencies = c.Nonredundant(Attributes, FunctionalDependencies)
  For Each FD1 As FunctionalDependency In FunctionalDependencies
   Dim Found As Boolean = False
   For Each FD2 As FunctionalDependency In ReducedFunctionalDependencies
    If FD1 Is FD2 Then
     Found = True
    End If
   Next
   If Found = False Then
    Dim strFD As String = ""
    For Each Attribute As Integer In FD1.LeftHandSide
     strFD += Attributes(Attribute) + " "
    Next
    strFD += "-> "
    For Each Attribute As Integer In FD1.RightHandSide
     strFD += Attributes(Attribute) + " "
    Next
    lstRedundantFDs.Items.Add(strFD)
   End If
  Next
  If lstRedundantFDs.Items.Count = 0 Then
   lstRedundantFDs.Items.Add("No redundant FD found.")
  End If
 End Sub

 'Form overrides dispose to clean up the component list.
 Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
  If disposing Then
   If Not (components Is Nothing) Then
    components.Dispose()
   End If
  End If
  MyBase.Dispose(disposing)
 End Sub

 'Required by the Windows Form Designer
 Private components As System.ComponentModel.IContainer

 'NOTE: The following procedure is required by the Windows Form Designer
 'It can be modified using the Windows Form Designer.  
 'Do not modify it using the code editor.
 Friend WithEvents lbl2NF As System.Windows.Forms.Label
 Friend WithEvents lstRedundantFDs As System.Windows.Forms.ListBox
 Friend WithEvents btnOK As System.Windows.Forms.Button
 <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
  Dim resources As System.Resources.ResourceManager = New System.Resources.ResourceManager(GetType(CheckRedundancy))
  Me.lbl2NF = New System.Windows.Forms.Label
  Me.btnOK = New System.Windows.Forms.Button
  Me.lstRedundantFDs = New System.Windows.Forms.ListBox
  Me.SuspendLayout()
  '
  'lbl2NF
  '
  Me.lbl2NF.Location = New System.Drawing.Point(7, 8)
  Me.lbl2NF.Name = "lbl2NF"
  Me.lbl2NF.Size = New System.Drawing.Size(226, 21)
  Me.lbl2NF.TabIndex = 2
  Me.lbl2NF.Text = "The following FDs are redundant:"
  Me.lbl2NF.TextAlign = System.Drawing.ContentAlignment.MiddleCenter
  '
  'btnOK
  '
  Me.btnOK.Location = New System.Drawing.Point(81, 416)
  Me.btnOK.Name = "btnOK"
  Me.btnOK.Size = New System.Drawing.Size(80, 24)
  Me.btnOK.TabIndex = 3
  Me.btnOK.Text = "OK"
  '
  'lstRedundantFDs
  '
  Me.lstRedundantFDs.Location = New System.Drawing.Point(8, 32)
  Me.lstRedundantFDs.Name = "lstRedundantFDs"
  Me.lstRedundantFDs.Size = New System.Drawing.Size(224, 368)
  Me.lstRedundantFDs.TabIndex = 4
  '
  'CheckRedundancy
  '
  Me.AutoScaleBaseSize = New System.Drawing.Size(5, 13)
  Me.ClientSize = New System.Drawing.Size(243, 444)
  Me.Controls.Add(Me.lstRedundantFDs)
  Me.Controls.Add(Me.btnOK)
  Me.Controls.Add(Me.lbl2NF)
  Me.FormBorderStyle = System.Windows.Forms.FormBorderStyle.Fixed3D
  Me.Icon = CType(resources.GetObject("$this.Icon"), System.Drawing.Icon)
  Me.MaximizeBox = False
  Me.MinimizeBox = False
  Me.Name = "CheckRedundancy"
  Me.Text = "Check Redundancy"
  Me.ResumeLayout(False)

 End Sub

#End Region

#End Region

#Region "Button-click event handlers"

 Private Sub btnOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnOK.Click
  If ReducedFunctionalDependencies.Count < FunctionalDependencies.Count Then
   If MessageBox.Show("Throw away redundant functional dependencies? (This will modify your current FD set.)", "", MessageBoxButtons.YesNo, MessageBoxIcon.Question) = DialogResult.Yes Then
    ReturnedFunctionalDependencies = ReducedFunctionalDependencies
   End If
  End If
  Me.Close()
 End Sub

#End Region

End Class

Public Class CheckRedundancyObject

#Region "Core functions"

 Public Function Nonredundant(ByVal UniversalAttributeSet As ArrayList, ByVal FDSet As ArrayList) As ArrayList
  Dim Lin As New LinClosure
  Dim NonredundantFDset As New ArrayList
  Dim FDtemp As FunctionalDependency
  Dim i, j, k As Integer
  Dim count As Integer
  For Each FD As FunctionalDependency In FDSet
   NonredundantFDset.Add(FD)
  Next
  i = 0
  While i < NonredundantFDset.Count
   count = 0
   FDtemp = NonredundantFDset(i)
   NonredundantFDset.RemoveAt(i)
   Dim LHSClosure As ArrayList = Lin.ComputeLinClosure(UniversalAttributeSet, FDtemp.LeftHandSide, NonredundantFDset)
   For k = 0 To FDtemp.RightHandSide.Count - 1
    For j = 0 To LHSClosure.Count - 1
     If (LHSClosure(j) = FDtemp.RightHandSide(k)) Then
      count = count + 1
     End If
    Next
   Next
   If count < FDtemp.RightHandSide.Count Then
    NonredundantFDset.Insert(i, FDtemp)
    i = i + 1
   End If
  End While
  Return NonredundantFDset
 End Function

#End Region

End Class
